@@ -2,16 +2,20 @@ package DateTime::Format::Natural;
 
 use strict;
 use warnings;
-use base qw(DateTime::Format::Natural::Base);
+use base qw(
+    DateTime::Format::Natural::Base
+    DateTime::Format::Natural::Helpers
+);
 use boolean qw(true false);
 
 use Carp qw(croak);
 use DateTime ();
-use Date::Calc qw(Day_of_Week check_date);
 use List::MoreUtils qw(all any);
 use Params::Validate ':all';
+use Scalar::Util qw(blessed);
+use Storable qw(dclone);
 
-our $VERSION = '0.75';
+our $VERSION = '0.77';
 
 validation_options(
     on_fail => sub
@@ -37,12 +41,23 @@ sub new
 
 sub _init
 {
-    my ($self, %opts) = @_;
+    my $self = shift;
+    my %opts = @_;
 
-    $self->{Format}        = $opts{format}        || 'd/m/y';
-    $self->{Lang}          = $opts{lang}          || 'en';
-    $self->{Prefer_future} = $opts{prefer_future} || false;
-    $self->{Time_zone}     = $opts{time_zone}     || 'floating';
+    my %presets = (
+        lang          => 'en',
+        format        => 'd/m/y',
+        prefer_future => false,
+        time_zone     => 'floating',
+    );
+    foreach my $opt (keys %presets) {
+        $self->{ucfirst $opt} = $presets{$opt};
+    }
+    foreach my $opt (keys %opts) {
+        if (defined $opts{$opt}) {
+            $self->{ucfirst $opt} = $opts{$opt};
+        }
+    }
     $self->{Opts}{daytime} = $opts{daytime};
 
     my $mod = __PACKAGE__.'::Lang::'.uc($self->{Lang});
@@ -81,12 +96,23 @@ sub _init_check
                     eval { DateTime::TimeZone->new(name => shift) };
                     !$@;
                 }
-            }
+            },
         },
         daytime => {
             type => HASHREF,
             optional => true,
         },
+        datetime => {
+            type => OBJECT,
+            optional => true,
+            callbacks => {
+                'valid object' => sub
+                {
+                    my $obj = shift;
+                    blessed($obj) && $obj->isa('DateTime');
+                }
+            },
+        },
     });
 }
 
@@ -94,8 +120,9 @@ sub _init_vars
 {
     my $self = shift;
 
-    delete $self->{modified};
-    delete $self->{postprocess};
+    foreach my $member (qw(modified postprocess)) {
+        delete $self->{$member};
+    }
 }
 
 sub parse_datetime
@@ -111,7 +138,7 @@ sub parse_datetime
     my %count; $count{$_}++ foreach @count;
 
     if (scalar keys %count == 1 && $count{(keys %count)[0]} == 2) {
-        if ($date_string =~ /^\S+\b\s+\b\S+/) {
+        if ($date_string =~ /^\S+\b \s+ \b\S+/x) {
             ($date_string, @{$self->{tokens}}) = split /\s+/, $date_string;
             $self->{count}{tokens} = 1 + scalar @{$self->{tokens}};
         }
@@ -166,7 +193,7 @@ sub parse_datetime
         if ($year > $century) { $century-- };
         if (length $year == 2) { $year = "$century$year" };
 
-        unless (check_date($year, $month, $day)) {
+        unless ($self->_check_date($year, $month, $day)) {
             $self->_set_failure;
             $self->_set_error("(invalid date)");
             return $self->_get_datetime_object;
@@ -177,19 +204,26 @@ sub parse_datetime
             month => $month,
             day   => $day,
         );
+        $self->{datetime}->truncate(to => 'day');
 
         $self->_set_valid_exp;
-        $self->_set_modified(1);
 
         if (@{$self->{tokens} || []}) {
             $self->{count}{tokens}--;
             $self->_unset_valid_exp;
-            $self->_unset_modified;
             $self->_process;
         }
+
+        if ($self->{duration}) {
+            %{$self->{formatted}} = (
+                year  => $year,
+                month => $month,
+                day   => $day,
+            );
+        }
     }
     else {
-        @{$self->{tokens}} = split ' ', $date_string;
+        @{$self->{tokens}} = split /\s+/, $date_string;
         $self->{data}->__init('tokens')->($self);
         $self->{count}{tokens} = scalar @{$self->{tokens}};
 
@@ -199,23 +233,54 @@ sub parse_datetime
     return $self->_get_datetime_object;
 }
 
+sub _params_init
+{
+    my $self = shift;
+    my $params = pop;
+
+    if (@_ > 1) {
+        validate(@_, { string => { type => SCALAR }});
+        my %opts = @_;
+        foreach my $opt (keys %opts) {
+            ${$params->{$opt}} = $opts{$opt};
+        }
+    }
+    else {
+        validate_pos(@_, { type => SCALAR });
+        (${$params->{string}}) = @_;
+    }
+}
+
 sub _parse_init
 {
     my $self = shift;
 
-    if (@_ > 1) {
-        validate(@_, { string => { type => SCALAR }});
-        my %opts             = @_;
-        $self->{Date_string} = $opts{string};
-        (undef)              = $opts{debug}; # legacy
-    }
-    else {
-        validate_pos(@_, { type => SCALAR });
-        ($self->{Date_string}) = @_;
-    }
+    $self->_params_init(@_, { string => \$self->{Date_string} });
+
+    my $set_datetime = sub
+    {
+        my ($method, $args) = @_;
+
+        if (exists $self->{Datetime} && $method eq 'now') {
+            $self->{datetime} = dclone($self->{Datetime});
+        }
+        else {
+            $self->{datetime} = DateTime->$method(
+                time_zone => $self->{Time_zone},
+                %$args,
+            );
+        }
+    };
 
     unless ($self->{running_tests}) {
-        $self->{datetime} = DateTime->now(time_zone => $self->{Time_zone});
+        if (exists $self->{formatted}) {
+            $set_datetime->('new', {
+                map { $_ => $self->{formatted}{$_} } qw(year month day)
+            });
+        }
+        else {
+            $set_datetime->('now', {});
+        }
     }
 
     $self->_init_vars;
@@ -224,26 +289,42 @@ sub _parse_init
     $self->_unset_error;
     $self->_unset_valid_exp;
     $self->_unset_trace;
-    $self->_unset_modified;
 }
 
 sub parse_datetime_duration
 {
     my $self = shift;
 
-    $self->_parse_init(@_);
-
+    my $duration_string;
+    $self->_params_init(@_, { string => \$duration_string });
     my $timespan_sep = $self->{data}->__timespan('literal');
 
-    my @date_strings = $self->{Date_string} =~ /$timespan_sep/i
-      ? split /\s+ $timespan_sep \s+/ix, $self->{Date_string}
-      : ($self->{Date_string});
+    my @date_strings = $duration_string =~ /$timespan_sep/i
+      ? do { $self->{duration} = true;
+             split /\s+ $timespan_sep \s+/ix, $duration_string }
+      : do { $self->{duration} = false;
+             ($duration_string) };
+
+    my $dt_now;
+    if (scalar @date_strings == 1
+         && $date_strings[0] =~ /^for\s+/i
+    ) {
+        $dt_now = $self->parse_datetime('now');
+    }
 
     my @queue;
     foreach my $date_string (@date_strings) {
         push @queue, $self->parse_datetime($date_string);
     }
 
+    if ($self->success && defined $dt_now) {
+        unshift @queue, $dt_now;
+    }
+
+    foreach my $member (qw(duration formatted)) {
+        delete $self->{$member};
+    }
+
     return @queue;
 }
 
@@ -272,13 +353,15 @@ sub trace
 
     return join "\n", @{$self->{trace}},
       map  { my $unit = $_; "$unit: $self->{modified}{$unit}" }
-      grep { $_ ne 'total' } keys %{$self->{modified}};
+      keys %{$self->{modified}};
 }
 
 sub _process
 {
     my $self = shift;
 
+    my %opts;
+
     if (!exists $self->{lookup}) {
         foreach my $keyword (keys %{$self->{data}->__grammar('')}) {
             push @{$self->{lookup}{scalar @{$self->{data}->__grammar($keyword)->[0]}}}, $keyword;
@@ -286,8 +369,6 @@ sub _process
     }
 
     PARSE: foreach my $keyword (@{$self->{lookup}{$self->{count}{tokens}} || []}) {
-        last if $self->_get_modified >= $self->{count}{tokens};
-
         my @grammar = @{$self->{data}->__grammar($keyword)};
         my $types = shift @grammar;
 
@@ -328,48 +409,81 @@ sub _process
                 my $i;
                 foreach my $check (@{$expression->[2]}) {
                     my @pos = @{$expression->[1][$i++]};
-                    $valid_expression &= $check->(\%regex_stack, @pos);
+                    my $error;
+                    $valid_expression &= $check->(\%regex_stack, \@pos, \$error);
+                    unless ($valid_expression) {
+                        $self->_set_error("($error)");
+                        last;
+                    }
                 }
             }
             if ($valid_expression) {
                 $self->_set_valid_exp;
-                my $i;
+                my $i = 0;
                 foreach my $positions (@{$expression->[3]}) {
                     my ($c, @values);
                     foreach my $pos (@$positions) {
-                        $values[$c++] = exists $regex_stack{$pos}
-                          ? $regex_stack{$pos}
-                          : ${$self->_token($pos)};
+                        my $index = ref $pos eq 'HASH' ? (keys %$pos)[0] : $pos;
+                        $values[$c++] = ref $pos
+                          ? $index eq 'VALUE'
+                            ? $pos->{$index}
+                            : $self->SUPER::_helper($pos->{$index}, $regex_stack{$index})
+                          : exists $regex_stack{$index}
+                            ? $regex_stack{$index}
+                            : ${$self->_token($index)};
                     }
-                    @values = map { defined $_ ? $_ : () } @values;
-                    my $meth = 'SUPER::'.$expression->[-1]->[$i++];
-                    $self->$meth(@values);
+                    my $meth = 'SUPER::'.$expression->[5]->[$i];
+                    $self->$meth(@values, $expression->[4]->[$i++]);
                 }
+                %opts = %{$expression->[6]};
                 last PARSE;
             }
         }
     }
 
-    $self->_post_process_options;
+    $self->_post_process(%opts);
 }
 
-sub _post_process_options
+sub _post_process
 {
     my $self = shift;
+    my %opts = @_;
 
-    if ($self->{Prefer_future}) {
-        my %modified = map { $_ => true } grep { $_ ne 'total' } keys %{$self->{modified}};
+    if (exists $opts{truncate_to}) {
+        $self->{datetime}->truncate(to => $opts{truncate_to});
+    }
 
-        if ($self->{count}{tokens} == 1
-            && (any { $self->{tokens}->[0] =~ /$_/i } @{$self->{data}->{weekdays_all}})
-            && scalar keys %modified == 1
-            && (exists $self->{modified}{day} && $self->{modified}{day} == 1
-            && Day_of_Week($self->{datetime}->year, $self->{datetime}->month, $self->{datetime}->day)
+    if ($self->{Prefer_future} &&
+        (exists $opts{prefer_future} && $opts{prefer_future})
+    ) {
+        my %modified = map { $_ => true } keys %{$self->{modified}};
+        my $token_contains = sub
+        {
+            my ($identifier) = @_;
+            return any {
+              my $data = $_;
+              any {
+                my $token = $_;
+                $token =~ /$data/i;
+              } @{$self->{tokens}}
+            } @{$self->{data}->{$identifier}};
+        };
+
+        if ((all { /^(?:second|minute|hour)$/ } keys %modified)
+            && (exists $self->{modified}{hour} && $self->{modified}{hour} == 1)
+            && (exists $self->{modified}{minute} && $self->{modified}{minute} == 1)
+            && $self->{datetime}->hour < DateTime->now(time_zone => $self->{Time_zone})->hour
+        ) {
+            $self->{postprocess}{day} = 1;
+        }
+        elsif ($token_contains->('weekdays_all')
+            && (exists $self->{modified}{day} && $self->{modified}{day} == 1)
+            && ($self->_Day_of_Week($self->{datetime}->year, $self->{datetime}->month, $self->{datetime}->day)
              < DateTime->now(time_zone => $self->{Time_zone})->wday)
         ) {
             $self->{postprocess}{day} = 7;
         }
-        elsif ((any { my $month = $_; any { $_ =~ /$month/i } @{$self->{tokens}} } @{$self->{data}->{months_all}})
+        elsif ($token_contains->('months_all')
             && (all { /^(?:day|month)$/ } keys %modified)
             && (exists $self->{modified}{month} && $self->{modified}{month} == 1)
             && (exists $self->{modified}{day}
@@ -385,7 +499,8 @@ sub _post_process_options
 
 sub _token
 {
-    my ($self, $pos) = @_;
+    my $self = shift;
+    my ($pos) = @_;
 
     my $str = '';
     my $token = $self->{tokens}->[0 + $pos];
@@ -395,7 +510,7 @@ sub _token
       : \$str;
 }
 
-sub _add_trace       { push @{$_[0]->{trace}}, (caller(1))[3] }
+sub _register_trace  { push @{$_[0]->{trace}}, (caller(1))[3] }
 sub _unset_trace     { @{$_[0]->{trace}} = ()                 }
 
 sub _get_error       { $_[0]->{error}         }
@@ -410,14 +525,10 @@ sub _get_valid_exp   { $_[0]->{valid_expression}         }
 sub _set_valid_exp   { $_[0]->{valid_expression} = true  }
 sub _unset_valid_exp { $_[0]->{valid_expression} = false }
 
-sub _get_modified    { $_[0]->{modified}{total} || 0     }
-sub _set_modified    { $_[0]->{modified}{total} += $_[1] }
-sub _unset_modified  { $_[0]->{modified}{total}  = 0     }
-
 sub _get_datetime_object
 {
     my $self = shift;
-    
+
     my $dt = DateTime->new(
         time_zone => $self->{datetime}->time_zone->name,
         year      => $self->{datetime}->year,
@@ -438,15 +549,18 @@ sub _get_datetime_object
 # solely for testing purpose
 sub _set_datetime
 {
-    my ($self, $year, $month, $day, $hour, $min, $sec, $tz) = @_;
+    my $self = shift;
+    my ($year, $month, $day, $hour, $min, $sec, $tz) = @_;
 
-    $self->{datetime} = DateTime->new(time_zone => $tz || 'floating',
-                                      year      => $year,
-                                      month     => $month,
-                                      day       => $day,
-                                      hour      => $hour,
-                                      minute    => $min,
-                                      second    => $sec);
+    $self->{datetime} = DateTime->new(
+        time_zone => $tz || 'floating',
+        year      => $year,
+        month     => $month,
+        day       => $day,
+        hour      => $hour,
+        minute    => $min,
+        second    => $sec
+    );
     $self->{running_tests} = true;
 }
 
@@ -491,9 +605,10 @@ Creates a new C<DateTime::Format::Natural> object. Arguments to C<new()> are opt
 not necessarily required.
 
  $parser = DateTime::Format::Natural->new(
+           datetime      => DateTime->new(...),
            lang          => 'en',
            format        => 'mm/dd/yy',
-           prefer_future => '[0|1]'
+           prefer_future => '[0|1]',
            time_zone     => 'floating',
            daytime       => { morning   => 06,
                               afternoon => 13,
@@ -503,6 +618,10 @@ not necessarily required.
 
 =over 4
 
+=item * C<datetime>
+
+Overrides the present now with a DateTime object provided.
+
 =item * C<lang>
 
 Contains the language selected, currently limited to C<en> (english).
@@ -611,6 +730,9 @@ valuable suggestions & patches:
  Alex Bowley
  Elliot Shank
  Anirvan Chatterjee
+ Michael Reddick
+ Christian Brink
+ Giovanni Pensa
 
 =head1 SEE ALSO
 
